import mock
import unittest
from generic_request_signer.response import Response

class ResponseTests(unittest.TestCase):

    sut_class = Response

    def setUp(self):
        self.raw_response = mock.Mock()
        self.raw_response.code = 500
        self.raw_response.read.return_value = "foobar"
        self.sut = self.sut_class(self.raw_response)
        self.json_loads_patch = mock.patch('json.loads')
        self.json_loads = self.json_loads_patch.start()

    def tearDown(self):
        self.json_loads = self.json_loads_patch.stop()

    def test_init_captures_incoming_response_in_raw_form(self):
        self.assertEqual(self.sut.raw_response, self.raw_response)

    def test_will_store_original_raw_response_as_none_by_default(self):
        self.assertEqual(self.sut.original_raw_response, None)

    def test_status_code_property_returns_raw_response_code(self):
        self.assertEqual(500, self.sut.status_code)

    def test_will_read_in_raw_response_when_original_is_still_none(self):
        self.sut.original_raw_response = None
        result = self.sut.read()
        self.assertEqual(result, "foobar")

    def test_will_return_cached_value_when_original_is_already_set(self):
        self.sut.original_raw_response = "glow"
        result = self.sut.read()
        self.assertEqual(result, "glow")

    def test_json_property_returns_empty_object_when_read_results_in_empty_string(self):
        with mock.patch.object(self.sut, 'read') as read:
            read.return_value = ''
            result = self.sut.json
        self.assertEqual(result, {})

    def test_json_property_invokes_json_loads_when_read_results_in_not_empty_string(self):
        with mock.patch.object(self.sut, 'read') as read:
            read.return_value = 'gotime'
            self.sut.json
        self.json_loads.assert_called_once_with('gotime')

    def test_json_property_returns_json_loads_result_when_read_results_in_not_empty_string(self):
        with mock.patch.object(self.sut, 'read') as read:
            read.return_value = 'gotime'
            result = self.sut.json
        self.assertEqual(result, self.json_loads.return_value)

    def test_is_successful_returns_false_when_status_is_500_based(self):
        self.sut.raw_response.code = 500
        self.assertFalse(self.sut.is_successful)

    def test_is_successful_returns_false_when_status_is_400_based(self):
        self.sut.raw_response.code = 400
        self.assertFalse(self.sut.is_successful)

    def test_is_successful_returns_false_when_status_is_200(self):
        self.sut.raw_response.code = 200
        self.assertTrue(self.sut.is_successful)

    def test_is_successful_returns_false_when_status_is_201(self):
        self.sut.raw_response.code = 201
        self.assertTrue(self.sut.is_successful)

    def test_is_successful_returns_false_when_status_is_206(self):
        self.sut.raw_response.code = 206
        self.assertTrue(self.sut.is_successful)

    def test_is_successful_returns_false_when_status_is_204(self):
        self.sut.raw_response.code = 204
        self.assertTrue(self.sut.is_successful)



import mock
import unittest
from generic_request_signer import backend


class BasicSettingsApiCredentialsBackendTests(unittest.TestCase):

    sut_class = backend.BasicSettingsApiCredentialsBackend

    def setUp(self):
        self.client = mock.Mock()
        self.sut = self.sut_class(self.client)

    def test_client_error_message_is_defined(self):
        self.assertEqual(self.sut.CLIENT_ERROR_MESSAGE, "Client implementations must define a `{0}` attribute")

    def test_client_settings_error_message_is_defined(self):
        self.assertEqual(self.sut.CLIENT_SETTINGS_ERROR_MESSAGE, "Settings must contain a `{0}` attribute")

    def test_init_captures_incoming_client(self):
        self.assertEqual(self.sut.client, self.client)

    def test_base_url_invokes_get_setting_with_domain_settings_name(self):
        with mock.patch.object(self.sut_class, 'get_setting') as get_setting:
            self.sut.base_url
        get_setting.assert_called_once_with('domain_settings_name')

    def test_base_url_invokes_get_setting_with_client_id_settings_name(self):
        with mock.patch.object(self.sut_class, 'get_setting') as get_setting:
            self.sut.client_id
        get_setting.assert_called_once_with('client_id_settings_name')

    def test_base_url_invokes_get_setting_with_private_key_settings_name(self):
        with mock.patch.object(self.sut_class, 'get_setting') as get_setting:
            self.sut.private_key
        get_setting.assert_called_once_with('private_key_settings_name')

    def test_get_setting_must_be_implemented(self):
        with self.assertRaises(NotImplementedError):
            self.sut.get_setting('foo')



import json


class Response(object):

    def __init__(self, response):
        self.original_raw_response = None
        self.raw_response = response

    @property
    def status_code(self):
        return self.raw_response.code

    def read(self):
        if self.original_raw_response is None:
            self.original_raw_response = self.raw_response.read()
        return self.original_raw_response

    @property
    def json(self):
        response_content = self.read()
        if response_content == '':
            return {}
        return json.loads(response_content)

    @property
    def is_successful(self):
        return self._evaluate_response_code_for_success(self.status_code)

    def _evaluate_response_code_for_success(self, response_code):
        return response_code // 100 == 2



from cStringIO import StringIO
import mock
import unittest
from urllib import urlencode
from collections import OrderedDict
from generic_request_signer import constants
from generic_request_signer.factory import SignedRequestFactory, json_encoding, default_encoding

class SignedRequestFactoryTests(unittest.TestCase):

    sut_class = SignedRequestFactory

    def setUp(self):
        self.method = 'GET'
        self.private_key = '1234'
        self.client_id = 'foobar'
        self.raw_data = {'some':'data'}
        self.sut = self.sut_class(self.method, self.client_id, self.private_key, self.raw_data)

    def test_init_captures_incoming_client_id(self):
        self.assertEqual(self.sut.client_id, self.client_id)

    def test_init_captures_incoming_http_method(self):
        self.assertEqual(self.sut.http_method, self.method)

    def test_init_captures_incoming_private_key(self):
        self.assertEqual(self.sut.private_key, self.private_key)

    def test_has_basic_boundary_defined(self):
        self.assertEqual(self.sut.boundary, "----WebKitFormBoundaryJQlIRhP93LbaDnCQ")

    def test_generates_hash_from_binary_data(self):
        attachment = StringIO('some pdf text from a file').read()
        self.sut.http_method = 'POST'
        self.sut.raw_data = {'file': attachment, 'data': {'filename': 'readme.pdf', 'guid': '1234'}}
        result = self.sut._data_or_multiform_post_data()
        self.assertEqual(result, {'binary_data': 'f9f316e8da9f562aec6661cb120fc230'})

    def test_returns_none_when_file_in_data_but_not_post_method(self):
        attachment = StringIO('some pdf text from a file').read()
        self.sut.http_method = 'PUT'
        self.sut.raw_data = {'file': attachment, 'data': {'filename': 'readme.pdf', 'guid': '1234'}}
        result = self.sut._data_or_multiform_post_data()
        self.assertEqual(result, None)

    def test_returns_none_when_post_but_no_file_in_data(self):
        self.sut.http_method = 'POST'
        self.sut.raw_data = {'data': {'filename': 'readme.pdf', 'guid': '1234'}}
        result = self.sut._data_or_multiform_post_data()
        self.assertEqual(result, None)

    @mock.patch('generic_request_signer.factory.file_encoding')
    def test_init_captures_content_type_encodings(self, file_encode):
        with mock.patch('generic_request_signer.factory.json_encoding') as json_encode:
            self.sut = self.sut_class(self.method, self.client_id, self.private_key, self.sut.raw_data)
        self.assertEqual(self.sut.content_type_encodings, {'application/json':json_encode, 'multipart/form-data': file_encode})

    def test_json_encoding_dumps_json_data(self):
        result = json_encoding(['foo', {'bar': ('baz', None, 1.0, 2)}])
        self.assertEqual(result, '["foo", {"bar": ["baz", null, 1.0, 2]}]')

    def test_default_encoding_encodes_url_data(self):
        result = default_encoding({'foo':'bar', 'baz':'broken'})
        self.assertEqual(result, 'foo=bar&baz=broken')

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_for_get_request_returns_url_with_data_client_id_and_signature(self, get_signature):
        get_signature.return_value = 'zzz123'
        self.sut.raw_data = {'username': u'some.user', 'token': u'813bc1ad91dfadsfsdfsd02c'}
        url = 'http://bit.ly/'
        self.sut.http_method = 'GET'
        self.sut.client_id = 'foobar'
        result = self.sut.build_request_url(url)
        self.assertEqual(result, 'http://bit.ly/?__client_id=foobar&username=some.user&token=813bc1ad91dfadsfsdfsd02c&__signature=zzz123')

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_for_post_request_returns_url_with_only_client_id_and_signature(self, get_signature):
        get_signature.return_value = 'zzz123'
        self.sut.raw_data = {'username': u'some.user', 'token': u'813bc1ad91dfadsfsdfsd02c'}
        url = 'http://bit.ly/'
        self.sut.http_method = 'POST'
        self.sut.client_id = 'foobar'
        result = self.sut.build_request_url(url)
        self.assertEqual(result, 'http://bit.ly/?__client_id=foobar&__signature=zzz123')

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_for_get_request_without_raw_data_returns_url_with_only_client_id_and_signature(self, get_signature):
        get_signature.return_value = 'zzz123'
        self.sut.raw_data = None
        url = 'http://bit.ly/'
        self.sut.http_method = 'GET'
        self.sut.client_id = 'foobar'
        result = self.sut.build_request_url(url)
        self.assertEqual(result, 'http://bit.ly/?__client_id=foobar&__signature=zzz123')

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_invokes_get_signature_with_no_data_when_get_request_and_valid_data(self, get_signature):
        url = 'http://bit.ly/'
        self.sut.raw_data = {'some':'data'}
        self.sut.http_method = 'GET'
        self.sut._build_signed_url(url)
        get_signature.assert_called_once_with(self.private_key, url, {})

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_invokes_get_signature_with_data_input_when_get_request_but_invalid_data(self, get_signature):
        url = 'http://bit.ly/'
        self.sut.raw_data = None
        self.sut.http_method = 'GET'
        self.sut._build_signed_url(url)
        get_signature.assert_called_once_with(self.private_key, url, None)

    @mock.patch('apysigner.get_signature')
    def test_build_signed_url_invokes_get_signature_with_raw_data_when_post_request(self, get_signature):
        url = 'http://bit.ly/'
        self.sut.http_method = 'POST'
        self.sut._build_signed_url(url)
        get_signature.assert_called_once_with(self.private_key, url, self.sut.raw_data)

    def test_will_modify_the_headers_content_type_to_include_boundary_when_multipart(self):
        self.sut.http_method = 'POST'
        self.sut.raw_data = {'file':''}
        headers = {'Content-Type':'multipart/form-data'}
        self.sut.modify_headers_when_multipart(headers)
        self.assertEqual('multipart/form-data; boundary={}'.format(self.sut.boundary), headers['Content-Type'])

    def test_will_not_modify_the_headers_content_type_when_http_method_is_not_post(self):
        self.sut.http_method = 'PUT'
        self.sut.raw_data = {'file':''}
        headers = {'Content-Type':'multipart/form-data'}
        self.sut.modify_headers_when_multipart(headers)
        self.assertEqual('multipart/form-data', headers['Content-Type'])

    def test_will_not_modify_the_headers_content_type_when_raw_data_has_no_file(self):
        self.sut.http_method = 'POST'
        self.sut.raw_data = {'no_binary_data_here':''}
        headers = {'Content-Type':'multipart/form-data'}
        self.sut.modify_headers_when_multipart(headers)
        self.assertEqual('multipart/form-data', headers['Content-Type'])

    def test_invokes_modify_headers_when_multipart_with_request_headers(self):
        self.sut.raw_data = {'file': 'foo', 'data': {'filename': 'readme.pdf', 'guid': '1234'}}
        headers = {'Content-Type':'multipart/form-data'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'modify_headers_when_multipart') as modify_headers:
            self.sut._get_data_payload(headers)
        modify_headers.assert_called_once_with(headers)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_invokes_get_on_internal_payload_when_raw_data_exists_and_not_get_request(self, default_encoding):
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'content_type_encodings') as encodings:
            self.sut._get_data_payload(headers)
        encodings.get.assert_called_once_with('application/json', default_encoding)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_constructs_func_with_raw_data_when_raw_data_exists_and_not_get_request(self, default_encoding):
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'content_type_encodings') as encodings:
            self.sut._get_data_payload(headers)
        encodings.get().assert_called_once_with(self.sut.raw_data, self.sut.boundary)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_returns_encoding_func_when_raw_data_exists_and_not_get_request(self, default_encoding):
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'content_type_encodings') as encodings:
            result = self.sut._get_data_payload(headers)
        self.assertEqual(result, encodings.get().return_value)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_does_not_invoke_get_on_internal_payload_when_no_raw_data_exists(self, default_encoding):
        self.sut.raw_data = None
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'content_type_encodings') as encodings:
            self.sut._get_data_payload(headers)
        self.assertFalse(encodings.get.called)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_returns_none_when_no_raw_data_exists(self, default_encoding):
        self.sut.raw_data = None
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'POST'
        with mock.patch.object(self.sut, 'content_type_encodings'):
            result = self.sut._get_data_payload(headers)
        self.assertEqual(result, None)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_does_not_invoke_get_on_internal_payload_when_http_get(self, default_encoding):
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'GET'
        with mock.patch.object(self.sut, 'content_type_encodings') as encodings:
            self.sut._get_data_payload(headers)
        self.assertFalse(encodings.get.called)

    @mock.patch('generic_request_signer.factory.default_encoding')
    def test_get_data_payload_returns_none_when_http_get(self, default_encoding):
        headers = {'Content-Type':'application/json'}
        self.sut.http_method = 'GET'
        with mock.patch.object(self.sut, 'content_type_encodings'):
            result = self.sut._get_data_payload(headers)
        self.assertEqual(result, None)

    def test_encodes_dict_of_data(self):
        result = default_encoding(OrderedDict((('a', 1), ('b', 2), ('c', 'asdf'))))
        self.assertEqual('a=1&b=2&c=asdf', result)

    def test_encodes_dict_with_nested_list(self):
        result = default_encoding(OrderedDict((('a', 1), ('b', [2, 4]), ('c', 'asdf'))))
        self.assertEqual('a=1&b=2&b=4&c=asdf', result)

    def test_encodes_dict_with_nested_empty_list(self):
        result = default_encoding(OrderedDict((('a', 1), ('b', []), ('c', 'asdf'))))
        self.assertEqual('a=1&c=asdf', result)

    @mock.patch('generic_request_signer.request.Request', mock.Mock)
    @mock.patch('generic_request_signer.factory.SignedRequestFactory._get_data_payload', mock.Mock)
    @mock.patch('generic_request_signer.factory.SignedRequestFactory.build_request_url')
    def test_create_request_invokes_build_request_url_with_params(self, build_url):
        url = '/foo/'
        self.sut.create_request(url)
        build_url.assert_called_once_with(url)

    @mock.patch('generic_request_signer.request.Request', mock.Mock)
    @mock.patch('generic_request_signer.factory.SignedRequestFactory.build_request_url', mock.Mock)
    @mock.patch('generic_request_signer.factory.SignedRequestFactory._get_data_payload')
    def test_create_request_invokes_get_data_payload_with_params(self, get_payload):
        kwargs = {'headers':'foo'}
        self.sut.create_request('/foo/', **kwargs)
        get_payload.assert_called_once_with(kwargs.get('headers', {}))

    @mock.patch('generic_request_signer.request.Request')
    @mock.patch('generic_request_signer.factory.SignedRequestFactory._get_data_payload')
    @mock.patch('generic_request_signer.factory.SignedRequestFactory.build_request_url')
    def test_create_request_build_actual_request_object_with_params(self, build_url, get_payload, request):
        url = '/foo/'
        args = {'random':'stuff'}
        kwargs = {'headers':'foo'}
        self.sut.http_method = 'GET'
        self.sut.create_request(url, *args, **kwargs)
        request.assert_called_once_with('GET', build_url.return_value, get_payload.return_value, *args, **kwargs)

    @mock.patch('generic_request_signer.factory.SignedRequestFactory._get_data_payload', mock.Mock)
    @mock.patch('generic_request_signer.factory.SignedRequestFactory.build_request_url', mock.Mock)
    @mock.patch('generic_request_signer.request.Request')
    def test_create_request_returns_request_object(self, request):
        result = self.sut.create_request('/foo/', {})
        self.assertEqual(result, request.return_value)

class LegacySignedRequestFactoryTests(unittest.TestCase):

    def setUp(self):
        self.client_id = 'client_id'
        self.private_key = 'oVB_b3qrP3R6IDApALqehQzFy3DpMfob6Y4627WEK5A='
        self.raw_data = {'some':'data'}
        self.sut = SignedRequestFactory('GET', self.client_id, self.private_key, self.raw_data)

    def test_sets_client_id_in_init(self):
        self.assertEqual(self.client_id, self.sut.client_id)

    def test_sets_private_key_in_init(self):
        self.assertEqual(self.private_key, self.sut.private_key)

    def test_adds_client_id_to_url(self):
        url = 'http://example.com/my/url'
        self.sut.raw_data = {}
        request = self.sut.create_request(url)

        querystring = "?{}={}".format(constants.CLIENT_ID_PARAM_NAME, self.client_id)
        querystring += "&{}={}".format(constants.SIGNATURE_PARAM_NAME, 'N1WOdyaBUVlPjKVyL3ionapOLAasFdvagfotfCdCW-Y=')
        self.assertEqual(url + querystring, request.get_full_url())

    def test_adds_signature_to_url(self):
        url = 'http://example.com/my/url'
        self.sut.raw_data = {}
        request = self.sut.create_request(url)

        querystring = "?{}={}".format(constants.CLIENT_ID_PARAM_NAME, self.client_id)
        querystring += "&{}={}".format(constants.SIGNATURE_PARAM_NAME, 'N1WOdyaBUVlPjKVyL3ionapOLAasFdvagfotfCdCW-Y=')
        self.assertEqual(url + querystring, request.get_full_url())

    @mock.patch('urllib2.Request.__init__')
    def test_urlencodes_data_as_part_of_url_when_method_is_get(self, urllib2_request):
        self.sut.raw_data = {'some':'da ta', 'goes':'he re'}
        self.sut.create_request('www.myurl.com')
        self.assertEqual(None, urllib2_request.call_args[0][1])
        url = "www.myurl.com?{}={}&some=da+ta&goes=he+re&{}={}".format(
            constants.CLIENT_ID_PARAM_NAME,
            self.client_id,
            constants.SIGNATURE_PARAM_NAME,
            '6dBfb4JhoJIm7FyzktbhFxBFyLBPTmXn-MLkV-RXLng='
        )
        self.assertEqual(url, urllib2_request.call_args[0][0])

    @mock.patch('urllib2.Request.__init__')
    def test_passes_data_to_urllib_request_when_method_is_not_get(self, urllib2_request):
        self.sut.raw_data = {'some': 'da ta', 'goes': 'he re'}
        self.sut.http_method = 'POST'
        self.sut.create_request('www.myurl.com')
        self.assertEqual(urlencode(self.sut.raw_data), urllib2_request.call_args[0][1])
        url = "www.myurl.com?{}={}&{}={}".format(
            constants.CLIENT_ID_PARAM_NAME,
            self.client_id,
            constants.SIGNATURE_PARAM_NAME,
            '3sh6DOlYgbsCGT5rNlY819eFAdfl6Fy9GiyHHgUAwLQ='
        )
        self.assertEqual(url, urllib2_request.call_args[0][0])

    @mock.patch('urllib2.Request.__init__')
    def test_passes_hash_from_binary_to_urllib_request_as_part_of_signature_when_multipart_form_post(self, request):
        attachment = StringIO('some pdf text from a file').read()
        self.sut.raw_data = {'file': attachment, 'data': {'filename': 'readme.pdf', 'guid': '1234'}}
        self.sut.http_method = 'POST'
        self.sut.create_request('www.myurl.com')
        #self.assertEqual(urlencode(self.sut.raw_data), urllib2_request.call_args[0][1])
        url = "www.myurl.com?{}={}&{}={}".format(
            constants.CLIENT_ID_PARAM_NAME,
            self.client_id,
            constants.SIGNATURE_PARAM_NAME,
            'UJ81GLdrVknMqfVgf88YCzGq_VhwWBfhv-sRD74b_O4='
        )
        self.assertEqual(url, request.call_args[0][0])

    def test_payload_is_empty_on_get_request_when_signed(self):
        url = "www.myurl.com?asdf=1234"
        self.sut.raw_data = {'asdf': '1234'}

        first_request = self.sut._build_signed_url(url)
        second_request = self.sut._build_signed_url(url)

        self.assertEqual(first_request, second_request)

    def test_get_data_payload_returns_none_when_no_raw_data(self):
        self.sut.raw_data = None
        payload_data = self.sut._get_data_payload({})
        self.assertEqual(None, payload_data)

    def test_get_data_payload_returns_none_when_get_request(self):
        self.sut.http_method = "GET"
        payload_data = self.sut._get_data_payload({})
        self.assertEqual(None, payload_data)

    def test_get_data_payload_returns_properly_encoded_data_when_content_type_header_present(self):
        encoder = mock.MagicMock()
        self.sut.http_method = "POST"
        self.sut.content_type_encodings = {"application/json": encoder}

        request_headers = {"Content-Type": "application/json"}
        payload_data = self.sut._get_data_payload(request_headers)
        self.assertEqual(encoder.return_value, payload_data)
        encoder.assert_called_once_with(self.sut.raw_data, self.sut.boundary)

    def test_get_data_payload_returns_default_encoded_data_when_no_content_type_header(self):
        self.sut.http_method = "POST"
        with mock.patch('generic_request_signer.factory.default_encoding') as encoder:
            payload_data = self.sut._get_data_payload(self.sut.raw_data)
        self.assertEqual(encoder.return_value, payload_data)
        encoder.assert_called_once_with(self.sut.raw_data, self.sut.boundary)

    def test_create_request_sends_header_data_to_get_data_payload(self):
        request_kwargs = {"headers": {"Content-Type": "application/json"}}
        with mock.patch.object(self.sut, "_get_data_payload") as get_payload:
            self.sut.create_request("/", **request_kwargs)
        get_payload.assert_called_once_with(request_kwargs["headers"])

    def test_create_request_sends_empty_dict_to_get_data_payload_when_no_header(self):
        with mock.patch.object(self.sut, "_get_data_payload") as get_payload:
            self.sut.create_request("/")
        get_payload.assert_called_once_with({})



class BasicSettingsApiCredentialsBackend(object):

    CLIENT_ERROR_MESSAGE = "Client implementations must define a `{0}` attribute"
    CLIENT_SETTINGS_ERROR_MESSAGE = "Settings must contain a `{0}` attribute"

    def __init__(self, client):
        self.client = client

    @property
    def base_url(self):
        return self.get_setting('domain_settings_name')

    @property
    def client_id(self):
        return self.get_setting('client_id_settings_name')

    @property
    def private_key(self):
        return self.get_setting('private_key_settings_name')

    def get_setting(self, name):
        raise NotImplementedError



import json
import request
import constants
import apysigner
from urllib import urlencode
from generic_request_signer.multipart_encoder import MultipartEncoder
from generic_request_signer.check_signature import generate_hash_for_binary

def default_encoding(raw_data, *args):
    return urlencode(raw_data, doseq=True)

def json_encoding(raw_data, *args):
    return json.dumps(raw_data)

def file_encoding(raw_data, *args):
    return MultipartEncoder(raw_data, *args).encode_multipart_formdata()

class SignedRequestFactory(object):

    boundary = "----WebKitFormBoundaryJQlIRhP93LbaDnCQ"

    def __init__(self, http_method, client_id, private_key, data):
        self.client_id = client_id
        self.private_key = private_key
        self.http_method = http_method
        self.raw_data = data
        self.content_type_encodings = {
            'application/json': json_encoding,
            'multipart/form-data': file_encoding,
            }

    def create_request(self, url, *args, **request_kwargs):
        url = self.build_request_url(url)
        data = self._get_data_payload(request_kwargs.get("headers", {}))
        return request.Request(self.http_method, url, data, *args, **request_kwargs)

    def build_request_url(self, url):
        url = self._build_client_url(url)
        if self._is_get_request_with_data():
            url += "&{0}".format(urlencode(self.raw_data))
        return self._build_signed_url(url)

    def _build_signed_url(self, url):
        data = self._get_data_based_on_http_method()
        signature = apysigner.get_signature(self.private_key, url, data)
        signed_url = url + "&{}={}".format(constants.SIGNATURE_PARAM_NAME, signature)
        return signed_url

    def _get_data_payload(self, request_headers):
        if self.raw_data and self.http_method.lower() != 'get':
            encoding_func = self.content_type_encodings.get(request_headers.get("Content-Type"), default_encoding)
            self.modify_headers_when_multipart(request_headers)
            return encoding_func(self.raw_data, self.boundary)

    def modify_headers_when_multipart(self, request_headers):
        if self.is_multipart_http_post:
            new_content_type = "{}; boundary={}".format(request_headers.get("Content-Type"), self.boundary)
            request_headers['Content-Type'] = new_content_type

    def _get_data_based_on_http_method(self):
        data = {} if self._is_get_request_with_data() else self.raw_data
        return self._data_or_multiform_post_data() or data

    def _is_get_request_with_data(self):
        return self.http_method.lower() == 'get' and self.raw_data

    def _data_or_multiform_post_data(self):
        if self.is_multipart_http_post:
            return generate_hash_for_binary(self.raw_data['file'])

    @property
    def is_multipart_http_post(self):
        return self.http_method.lower() == 'post' and 'file' in self.raw_data.keys()

    def _build_client_url(self, url):
        url += "?%s=%s" % (constants.CLIENT_ID_PARAM_NAME, self.client_id)
        return url


import urllib2
from exceptions import HttpMethodNotAllowed


class Request(urllib2.Request, object):

    http_method_names = ['get', 'post', 'put', 'delete', 'head', 'options', 'trace']

    def __init__(self, http_method, url, data, *args, **kwargs):
        method_lower = http_method.lower()
        if method_lower not in self.http_method_names:
            raise HttpMethodNotAllowed
        self.http_method = http_method
        super(Request, self).__init__(url, data, *args, **kwargs)

    def get_method(self):
        return self.http_method



import unittest
from generic_request_signer.exceptions import WebException, HttpMethodNotAllowed


class ExceptionsTests(unittest.TestCase):

    def test_web_exception_is_subclass_of_exception(self):
        self.assertTrue(issubclass(WebException, Exception))

    def test_http_method_not_allowed_is_subclass_of_exception(self):
        self.assertTrue(issubclass(HttpMethodNotAllowed, Exception))



import urllib2
import response
import factory


class Client(object):

    def __init__(self, api_credentials):
        self.api_credentials = api_credentials

    def _get_response(self, http_method, endpoint, data=None, **request_kwargs):
        try:
            http_response = urllib2.urlopen(self._get_request(http_method, endpoint, data, **request_kwargs))
        except urllib2.HTTPError as e:
            http_response = e
        return response.Response(http_response)

    def _get_request(self, http_method, endpoint, data=None, **request_kwargs):
        request_factory = factory.SignedRequestFactory(http_method, self._client_id, self._private_key, data)
        service_url = self._get_service_url(endpoint)
        return request_factory.create_request(service_url, **request_kwargs)

    def _get_service_url(self, endpoint):
        return self._base_url + endpoint

    @property
    def _base_url(self):
        return self.api_credentials.base_url

    @property
    def _client_id(self):
        return self.api_credentials.client_id

    @property
    def _private_key(self):
        return self.api_credentials.private_key



class MultipartEncoder(object):

    def __init__(self, raw_data, boundary):
        self.raw_data = raw_data
        self.boundary = boundary
        self.body = []

    def encode_multipart_formdata(self):
        for k,v in self.raw_data.items():
            self.body.append(self._generate_multipart_post_body(k))
        self.body.append('--{}--\r\n'.format(self.boundary))
        return '\r\n'.join(self.body)

    def _generate_multipart_post_body(self, key):
        if key == 'file':
            return self._create_multipart_body_element(self.raw_data, key)
        else:
            return self._generate_post_data_body(key)

    def _generate_post_data_body(self, key):
        post_data = []
        for k,v in self.raw_data[key].items():
            post_data.append(self._create_multipart_body_element(self.raw_data[key], k))
        return '\r\n'.join(post_data)

    def _create_multipart_body_element(self, data, key):
        boundary = '--{}'.format(self.boundary)
        disposition = self._generate_content_disposition(key)
        value = data[key]
        return "{}\r\n{}\r\n{}\r\n".format(boundary, disposition, value)

    def _generate_content_disposition(self, key):
        disposition = 'Content-Disposition: form-data; name="{}"{}\r\n'
        return disposition.format('file','; filename="file"') if key == 'file' else disposition.format(key,'')


import mock
import urllib2
import unittest
from generic_request_signer import client


class ClientTests(unittest.TestCase):

    sut_class = client.Client

    def setUp(self):
        self.api_credentials = FakeApiCredentials('/foo', '1', 'bar')
        self.sut = self.sut_class(self.api_credentials)
        self.urlopen_patch = mock.patch('urllib2.urlopen')
        self.urlopen = self.urlopen_patch.start()
        self.response_patch = mock.patch('generic_request_signer.response.Response')
        self.response = self.response_patch.start()

    def tearDown(self):
        self.urlopen_patch.stop()
        self.response_patch.stop()

    def test_init_captures_incoming_api_credentials(self):
        self.assertEqual(self.sut.api_credentials, self.api_credentials)

    def test_base_url_property_returns_api_credentials_base_url_value(self):
        self.assertEqual(self.sut._base_url, '/foo')

    def test_client_id_property_returns_api_credentials_client_id_value(self):
        self.assertEqual(self.sut._client_id, '1')

    def test_private_key_property_returns_api_credentials_private_key_value(self):
        self.assertEqual(self.sut._private_key, 'bar')

    def test_get_service_url_returns_valid_endpoint(self):
        self.assertEqual(self.sut._get_service_url('/bazz/'), '/foo/bazz/')

    def test_get_response_invokes_urlopen_with_request_return_value(self):
        with mock.patch.object(self.sut_class, '_get_request') as get_request:
            self.sut._get_response('GET', '/', {}, **{})
        self.urlopen.assert_called_once_with(get_request.return_value)

    def test_get_response_invokes_get_request_incoming_params(self):
        method = 'GET'
        endpoint = '/'
        data = {'some':'data'}
        kwargs = {'some':'kwarg'}
        with mock.patch.object(self.sut_class, '_get_request') as get_request:
            self.sut._get_response(method, endpoint, data, **kwargs)
        get_request.assert_called_once_with(method, endpoint, data, **kwargs)

    def test_get_response_instantiates_response_with_urlopen_result(self):
        with mock.patch.object(self.sut_class, '_get_request'):
            self.sut._get_response('GET', '/', {}, **{})
        self.response.assert_called_once_with(self.urlopen.return_value)

    def test_when_urlopen_throws_exception_the_http_error_is_used_to_instantiate_the_response(self):
        http_error = urllib2.HTTPError('/',500,'',None,None)
        self.urlopen.side_effect = http_error
        with mock.patch.object(self.sut_class, '_get_request'):
            self.sut._get_response('GET', '/', {}, **{})
        self.response.assert_called_once_with(http_error)

    def test_get_response_returns_urlopen_result_in_http_response_object(self):
        with mock.patch.object(self.sut_class, '_get_request'):
            result = self.sut._get_response('GET', '/', {}, **{})
        self.assertEqual(result, self.response.return_value)

    def test_get_request_instantiates_factory_with_params(self):
        with mock.patch('generic_request_signer.factory.SignedRequestFactory') as factory:
            self.sut._get_request('GET', '/', {}, **{})
        factory.assert_called_once_with('GET', '1', 'bar', {})

    def test_get_request_invokes_create_request_on_factory(self):
        data = {'some':'data'}
        with mock.patch('generic_request_signer.factory.SignedRequestFactory') as factory:
            self.sut._get_request('GET', '/endpoint/', data, **{})
        factory.return_value.create_request.assert_called_once_with('/foo/endpoint/')

    def test_get_request_returns_factory_create_request_value(self):
        with mock.patch('generic_request_signer.factory.SignedRequestFactory') as factory:
            result = self.sut._get_request('GET', '/', {}, **{})
        self.assertEqual(result, factory.return_value.create_request.return_value)


class FakeApiCredentials(object):

    def __init__(self, base_url, client_id, private_key):
        self.base_url = base_url
        self.client_id = client_id
        self.private_key = private_key



import mock
import unittest
import base64
from generic_request_signer import constants, check_signature


class CheckSignatureForBinaryTests(unittest.TestCase):

    @mock.patch('generic_request_signer.check_signature.check_signature')
    def test_check_signature_for_binary_invokes_check_signature_with_hash_of_binary(self, check_signature_mock):
        payload = 'foobar'
        url_path = "/example/post-it/"
        private_key = 'UHJpdmF0ZSBLZXk='
        signature = "AKm9ZGCZPaYRMnLxpFZ8-ulaCIr_wKYAruZVm36uv3Q="
        expected_payload = {'binary_data':'3858f62230ac3c915f300c664312c63f'}
        check_signature.check_signature_for_binary(signature, private_key, url_path, payload)
        check_signature_mock.assert_called_once_with(signature, private_key, url_path, expected_payload)

class CheckSignatureTests(unittest.TestCase):

    def setUp(self):
        self.TEST_PRIVATE_KEY = 'UHJpdmF0ZSBLZXk='
        self.url_one = "/example/get-it/?q=find"
        self.url_two = "/example/post-it/"
        self.url_post_data = {'q': 'update'}

        # valid signatures for urls above and data
        self.signature_one = "T-lT3uT2wpUobJvDkXpxtsEAl7KmrEg6k3So_Varya8="
        self.signature_two = "AKm9ZGCZPaYRMnLxpFZ8-ulaCIr_wKYAruZVm36uv3Q="

    def test_returns_true_when_private_key_built_from_base64_url_encoded_string(self):
        base64_private_key = base64.urlsafe_b64encode('Private Key')
        signature_valid = check_signature.check_signature(self.signature_one, base64_private_key, self.url_one, None)
        self.assertEqual(True, signature_valid)

    def test_returns_true_when_signatures_match_and_no_post_data(self):
        signature_valid = check_signature.check_signature(self.signature_one, self.TEST_PRIVATE_KEY, self.url_one, None)
        self.assertEqual(True, signature_valid)

    def test_returns_true_when_signatures_match_and_has_post_data(self):
        signature_valid = check_signature.check_signature(self.signature_two, self.TEST_PRIVATE_KEY, self.url_two, self.url_post_data)
        self.assertEqual(True, signature_valid)

    def test_returns_false_when_signatures_dont_match(self):
        bad_signature = "ABCc0hS02rVC3016krevud1aW9B6Ls0Tp4_XcezuXYZ="
        signature_valid = check_signature.check_signature(bad_signature, self.TEST_PRIVATE_KEY, self.url_one, None)
        self.assertEqual(False, signature_valid)

    def test_doesnt_use_signature_already_in_url_to_check_for_valid_signature(self):
        url_with_sig = "{url}&{signature_param}={signature}".format(
            url=self.url_one,
            signature_param=constants.SIGNATURE_PARAM_NAME,
            signature=self.signature_one,
        )

        signature_valid = check_signature.check_signature(self.signature_one, self.TEST_PRIVATE_KEY, url_with_sig, None)
        self.assertEqual(True, signature_valid)

    def test_doesnt_use_signature_already_in_url_to_check_for_valid_signature_with_post_data(self):
        url_with_sig = "{url}?{signature_param}={signature}".format(
            url=self.url_two,
            signature_param=constants.SIGNATURE_PARAM_NAME,
            signature=self.signature_two,
        )

        signature_valid = check_signature.check_signature(self.signature_two, self.TEST_PRIVATE_KEY, url_with_sig, self.url_post_data)
        self.assertEqual(True, signature_valid)


class StripSignatureFromUrlTests(unittest.TestCase):

    def test_strip_signature_from_query_string_after_amp(self):
        url_path = '/api/CheckAuthentication/?__client_id=config&username=None&token=None&__signature=eWDBUhizESZkHtnyE807XbwdvivZKAIfwRltzcuOqgY='
        signature = 'eWDBUhizESZkHtnyE807XbwdvivZKAIfwRltzcuOqgY='
        result = check_signature._strip_signature_from_url(signature, url_path)
        clean_url = '/api/CheckAuthentication/?__client_id=config&username=None&token=None'
        self.assertEqual(result, clean_url)

    def test_strip_signature_from_query_string_with_no_amp(self):
        url_path = '/api/CheckAuthentication/?__signature=eWDBUhizESZkHtnyE807XbwdvivZKAIfwRltzcuOqgY='
        signature = 'eWDBUhizESZkHtnyE807XbwdvivZKAIfwRltzcuOqgY='
        result = check_signature._strip_signature_from_url(signature, url_path)
        self.assertEqual(result, '/api/CheckAuthentication/')


class ConstantTimeCompareTests(unittest.TestCase):

    def test_constant_time_compare(self):
    # It's hard to test for constant time, just test the result.
        self.assertTrue(check_signature.constant_time_compare(b'spam', b'spam'))
        self.assertFalse(check_signature.constant_time_compare(b'spam', b'eggs'))
        self.assertTrue(check_signature.constant_time_compare('spam', 'spam'))
        self.assertFalse(check_signature.constant_time_compare('spam', 'eggs'))



import unittest
from generic_request_signer.multipart_encoder import MultipartEncoder

class MultipartEncoderTests(unittest.TestCase):

    sut_class = MultipartEncoder

    def setUp(self):
        self.boundary = '--foo'
        self.raw_data = {'file': 'binary_value', 'data': {'name': 'readme.pdf', 'guid': '1234'}}
        self.sut = self.sut_class(self.raw_data, self.boundary)

    def test_generate_multipart_post_body_returns_string_with_filename_when_key_is_file(self):
        result = self.sut._generate_multipart_post_body('file')
        expected = '----foo\r\nContent-Disposition: form-data; name="file"; filename="file"\r\n\r\nbinary_value\r\n'
        self.assertEqual(result, expected)

    def test_generate_multipart_post_body_returns_string_without_filename_when_key_is_data(self):
        result = self.sut._generate_multipart_post_body('data')
        expected = '----foo\r\nContent-Disposition: form-data; name="guid"\r\n\r\n1234\r\n\r\n----foo\r\nContent-Disposition: form-data; name="name"\r\n\r\nreadme.pdf\r\n'
        self.assertEqual(result, expected)

    def test_encode_multipart_form_data_returns_formatted_string_with_all_elements_of_multipart_post(self):
        result = self.sut.encode_multipart_formdata()
        expected = '----foo\r\nContent-Disposition: form-data; name="guid"\r\n\r\n1234\r\n\r\n----foo\r\nContent-Disposition: form-data; name="name"\r\n\r\nreadme.pdf\r\n\r\n----foo\r\nContent-Disposition: form-data; name="file"; filename="file"\r\n\r\nbinary_value\r\n\r\n----foo--\r\n'
        self.assertEqual(result, expected)

    def test_create_multipart_body_element_creates_formatted_part_given_data_dictionary_and_key(self):
        result = self.sut._create_multipart_body_element(self.raw_data['data'], 'name')
        expected = '----foo\r\nContent-Disposition: form-data; name="name"\r\n\r\nreadme.pdf\r\n'
        self.assertEqual(result, expected)

    def test_generate_content_disposition_returns_name_of_file_if_key_is_file(self):
        result = self.sut._generate_content_disposition('file')
        expected = 'Content-Disposition: form-data; name="file"; filename="file"\r\n'
        self.assertEqual(result, expected)

    def test_generate_content_disposition_returns_name_of_key_if_key_is_not_file(self):
        result = self.sut._generate_content_disposition('name')
        expected = 'Content-Disposition: form-data; name="name"\r\n'
        self.assertEqual(result, expected)



import hashlib
import apysigner
import constants
import re


def generate_hash_for_binary(binary_data):
    return {'binary_data': hashlib.md5(binary_data).hexdigest()}

def check_signature_for_binary(signature, private_key, full_path, binary):
    binary_hash = generate_hash_for_binary(binary)
    return check_signature(signature, private_key, full_path, binary_hash)

def check_signature(signature, private_key, full_path, payload):
    """
    Checks signature received and verifies that we are able to re-create
    it from the private key, path, and payload given.

    :param signature:
        Signature received from request.
    :param private_key:
        Base 64, url encoded private key.
    :full_path:
        Full path of request, including GET query string (excluding host)
    :payload:
        The request.POST data if present. None if not.

    :returns:
        Boolean of whether signature matched or not.
    """
    url_to_check = _strip_signature_from_url(signature, full_path)
    computed_signature = apysigner.get_signature(private_key, url_to_check, payload)
    return constant_time_compare(signature, computed_signature)


def _strip_signature_from_url(signature, url_path):
    signature_qs = r"(\?|&)?{0}={1}$".format(constants.SIGNATURE_PARAM_NAME, signature)
    clean_url = re.sub(signature_qs, '', url_path, count=1)
    return clean_url


def constant_time_compare(val1, val2):
    """
    **This code was taken from the django 1.4.x codebase along with the test code**
    Returns True if the two strings are equal, False otherwise.

    The time taken is independent of the number of characters that match.
    """
    if len(val1) != len(val2):
        return False
    result = 0
    for x, y in zip(val1, val2):
        result |= ord(x) ^ ord(y)
    return result == 0


import mock
import unittest
from generic_request_signer.request import Request
from generic_request_signer.exceptions import HttpMethodNotAllowed

class RequestTests(unittest.TestCase):

    sut_class = Request

    def test_urllib2_super_invoked_with_params(self):
        url = '/'
        data = {}
        args = {'some':'args'}
        kwargs = {'more':'kwargs'}
        with mock.patch('urllib2.Request.__init__') as init:
            self.sut_class('GET', url, data, *args, **kwargs)
        init.assert_called_once_with(url, data, *args, **kwargs)

    def test_init_captures_incoming_http_method(self):
        with mock.patch('urllib2.Request.__init__'):
            sut = self.sut_class('GET', '/', {})
        self.assertEqual(sut.http_method, 'GET')

    def test_get_http_method_returns_correct_method(self):
        sut = self.sut_class('GET', '/', {})
        self.assertEqual(sut.get_method(), 'GET')

    def test_will_raise_exception_when_http_method_not_allowed(self):
        with self.assertRaises(HttpMethodNotAllowed):
            self.sut_class('HUH', '/', {})

    def test_will_not_raise_exception_when_http_method_is_allowed(self):
        with self.assertRaises(AssertionError):
            with self.assertRaises(HttpMethodNotAllowed):
                self.sut_class('POST', '/', {})

    def test_has_specific_set_of_http_methods_allowed(self):
        allowed_http_methods = ['get', 'post', 'put', 'delete', 'head', 'options', 'trace']
        self.assertEqual(allowed_http_methods, self.sut_class.http_method_names)
